<?php



/** All slugs*/

// Register a custom REST API endpoint

add_action('rest_api_init', function () {

    register_rest_route('wp/v2', '/content-urls', array(

        'methods' => 'GET',

        'callback' => 'get_all_content_urls_api',

        'permission_callback' => '__return_true', // Set appropriate permissions here

    ));

});



function get_all_content_urls_api() {

    // Initialize an empty array to store details

    $all_details = array();



    // Get home page and blog page IDs

    $home_page_id = get_option('page_on_front');

    $blog_page_id = get_option('page_for_posts');

    

    // Get all public post types (including custom post types)

    $post_types = get_post_types(array('public' => true), 'names');

    

    // Loop through each post type

    foreach ($post_types as $post_type) {

        // Get all posts of the current post type

        $posts = get_posts(array(

            'post_type'      => $post_type,

            'post_status'    => 'publish',

            'posts_per_page' => -1,

            'exclude'        => array($home_page_id, $blog_page_id) // Exclude home and blog pages

        ));

        

        // Loop through each post and get its details

        foreach ($posts as $post) {

            // Get the post template

            $template = get_page_template_slug($post->ID);

            if (empty($template)) {

                $template = 'default';

            }



            // Get parent page slug and full path name

            $parent_slug = '';

            if ($post->post_parent) {

                $parent_slug = get_post_field('post_name', $post->post_parent);

            }



            $path_name = build_full_path($post->ID);

            if (is_post_publicly_viewable(($post->ID))) {                

                $all_details[] = array(

                    'id'          => $post->ID,

                    'title'       => $post->post_title,

                    'slug'        => $post->post_name,

                    'parent_slug' => $parent_slug,

                    'template'    => $template,

                    'type'        => $post->post_type,

                    'path_name'   => $path_name,

                    'last_modified'   => get_the_modified_time('c', $post->ID)

                );

            }

        }

    }

    // Get home page details

    if ($home_page_id) {

        $home_page = get_post($home_page_id); // Retrieve the home page post object

        $home_page_template = get_page_template_slug($home_page_id);

        if (empty($home_page_template)) {

            $home_page_template = 'default';

        }



        // Get parent page slug and full path name for home page

        $home_parent_slug = '';

        if ($home_page->post_parent) {

            $home_parent_slug = get_post_field('post_name', $home_page->post_parent);

        }



        $home_path_name = build_full_path($home_page_id);



        $all_details[] = array(

            'id'          => $home_page_id,

            'title'       => $home_page->post_title,

            'slug'        => $home_page->post_name,

            'parent_slug' => $home_parent_slug,

            'template'    => $home_page_template,

            'type'        => 'home_page',

            'path_name'   => $home_path_name,

            'last_modified' => get_the_modified_time('c', $home_page_id)

        );

        

    }



    // Get blog page details

    if ($blog_page_id) {

        $blog_page = get_post($blog_page_id); // Retrieve the blog page post object

        $blog_page_template = get_page_template_slug($blog_page_id);

        if (empty($blog_page_template)) {

            $blog_page_template = 'default';

        }



        // Get parent page slug and full path name for blog page

        $blog_parent_slug = '';

        if ($blog_page->post_parent) {

            $blog_parent_slug = get_post_field('post_name', $blog_page->post_parent);

        }



        $blog_path_name = build_full_path($blog_page_id);



        $all_details[] = array(

            'id'          => $blog_page_id,

            'title'       => $blog_page->post_title,

            'slug'        => $blog_page->post_name,

            'parent_slug' => $blog_parent_slug,

            'template'    => $blog_page_template,

            'type'        => 'blog_page',

            'path_name'   => $blog_path_name,

            'last_modified' => get_the_modified_time('c', $blog_page_id)

        );

    }

    // Get all taxonomy terms

    $taxonomies = get_taxonomies(array('public' => true), 'objects');

    foreach ($taxonomies as $taxonomy) {

        $terms = get_terms(array(

            'taxonomy' => $taxonomy->name,

            'hide_empty' => false,

        ));

        foreach ($terms as $term) {

            if ($term->slug !== 'uncategorized') {

                $term_path_name = $taxonomy->name . '/' . $term->slug;

                $all_details[] = array(

                    'id' => $term->term_id,

                    'title' => $term->name,

                    'slug' => $term->slug,

                    'parent_slug' => $term->parent ? get_term($term->parent)->slug : '',

                    'template' => 'term',

                    'type' => 'taxonomy_term',

                    'taxonomy_name'=>$taxonomy->label,

                    'path_name' => $term_path_name,

                    'last_modified' => get_term_modified_date('c', $term->term_id)

                );

            }

        }

    }

    return new WP_REST_Response($all_details, 200);

}


function build_full_path($post_id) {
    $post = get_post($post_id);
    $path = [];

    // Get the custom post type slug and add it first
    $post_type_obj = get_post_type_object($post->post_type);
    if (!empty($post_type_obj->rewrite['slug'])) {
        $path[] = $post_type_obj->rewrite['slug']; // Add the post type slug first
    }

    // Build the path based on parent hierarchy
    while ($post_id) {
        $post = get_post($post_id);
        $path[] = $post->post_name; // Add the post slug and parent slugs
        $post_id = $post->post_parent;
    }

    return implode('/', $path);
}


// function build_full_path($post_id) {

//     $path = array();

//     while ($post_id) {

//         $post = get_post($post_id);

//         array_unshift($path, $post->post_name);

//         $post_id = $post->post_parent;

//     }

//     return implode('/', $path);

// }

function get_term_modified_date($format, $term_id) {

    $term = get_term($term_id);

    if ($term && $term->term_id) {

        $last_mod_date = get_term_meta($term_id, 'last_modified', true);

        if (!$last_mod_date) {

            $last_mod_date = current_time($format);

            update_term_meta($term_id, 'last_modified', $last_mod_date);

        }

        return date($format, strtotime($last_mod_date));

    }

    return '';

}



/**ACF Options */

// Hook into the REST API initialization action

add_action('rest_api_init', function () {

    // Check if ACF PRO is installed

    if (function_exists('acf_get_field_groups') && function_exists('acf_get_field_group') && function_exists('acf_get_fields') && function_exists('get_field')) {

        // Register the route for retrieving all ACF options page field groups

        register_rest_route('wp/v2', '/acf-options', array(

            'methods' => 'GET',

            'callback' => 'acf_options_callback',

            'permission_callback' => '__return_true', // Set appropriate permissions here

        ));



        // Register the route for retrieving fields of a specific group by group key

        register_rest_route('wp/v2', '/acf-options/(?P<group_key>[\w-]+)', array(

            'methods' => 'GET',

            'callback' => 'acf_options_group_callback',

            'permission_callback' => '__return_true', // Set appropriate permissions here

        ));



        // Register the route for retrieving a specific field value by field name and group key

        register_rest_route('wp/v2', '/acf-options/(?P<group_key>[\w-]+)/(?P<field_name>[\w-]+)', array(

            'methods' => 'GET',

            'callback' => 'acf_options_field_callback',

            'permission_callback' => '__return_true', // Set appropriate permissions here

        ));

    }

});



/**

 * Callback function to retrieve all ACF options page field groups

 */

function acf_options_callback() {

    $field_groups = acf_get_field_groups(array(

        'options_page' => true,

    ));



    if ($field_groups) {

        $all_details = [];

        foreach ($field_groups as $field_group) {

            $all_details[] = array(

                'id'    => $field_group['ID'],

                'key'   => $field_group['key'],

                'title' => $field_group['title'],

            );

        }

        return new WP_REST_Response($all_details, 200);

    } else {

        return new WP_REST_Response(array('message' => 'No ACF options page field groups found.'), 404);

    }

}



/**

 * Callback function to retrieve all fields and their values for a specific group by group key

 */

function acf_options_group_callback($data) {

    $group_key = $data['group_key'];

    $field_group = acf_get_field_group($group_key);



    if ($field_group) {

        $fields = acf_get_fields($group_key);



        if ($fields) {

            $all_details = [];

            foreach ($fields as $field) {

                if (isset($field['name']) && !empty($field['name'])) {

                    $field_value = get_field($field['name'], 'option');

                    if ($field_value !== false) {

                        $all_details[$field['name']] = $field_value;

                    }

                }

            }

            return new WP_REST_Response($all_details, 200);

        } else {

            return new WP_REST_Response(array('message' => 'No ACF fields found for the specified group key.'), 404);

        }

    } else {

        return new WP_REST_Response(array('message' => 'ACF field group not found for the specified group key.'), 404);

    }

}



/**

 * Callback function to retrieve the value of a specific field by field name and group key

 */

function acf_options_field_callback($data) {

    $field_name = $data['field_name'];



    if ($field_name) {

        $field_value = get_field($field_name, 'option');

        return new WP_REST_Response($field_value, 200);

    } else {

        return new WP_REST_Response(array('message' => 'No ACF field found for the specified field name.'), 404);

    }

}





/** Social Feeds  */

// Register custom REST API endpoints

add_action( 'rest_api_init', function () {

    register_rest_route( 'wp/v2', '/social-feeds', array(

        'methods' => 'GET',

        'callback' => 'get_all_feeds',

    ) );



    register_rest_route( 'wp/v2', '/social-feeds/(?P<id>.+)', array(

        'methods' => 'GET',

        'callback' => 'get_feed_posts',

        'permission_callback' => '__return_true'

    ) );

});



// Callback function to list all feed IDs

function get_all_feeds() {

    global $wpdb;



    // Query the wp_sbi_feeds table to get all feed IDs

    $feeds = $wpdb->get_results( "SELECT id,feed_name FROM {$wpdb->prefix}sbi_feeds" );

    if ( empty( $feeds ) ) {

        return new WP_Error( 'no_feeds', 'No feeds found', array( 'status' => 404 ) );

    }

    return rest_ensure_response( $feeds );

}



// Callback function to get posts of a specific feed ID with media_id and custom image path

function get_feed_posts( $data ) {

    global $wpdb;



    $feed_id = '*'.$data['id'];



    // Query the wp_sbi_instagram_feeds_posts table to get Instagram IDs for the specific feed ID

    $post_ids = $wpdb->get_results( 

        $wpdb->prepare( "SELECT instagram_id FROM {$wpdb->prefix}sbi_instagram_feeds_posts WHERE feed_id = %s", $feed_id )

    );



    if ( empty( $post_ids ) ) {

        return new WP_Error( 'no_posts', 'No posts found for this feed', array( 'status' => 404 ) );

    }



    // Extract instagram_ids

    $instagram_ids = array();

    foreach ( $post_ids as $post ) {

        $instagram_ids[] = $post->instagram_id;

    }



    // Query the wp_sbi_instagram_posts table to get post details (media_id and generate custom image path)

    $placeholders = implode( ',', array_fill( 0, count( $instagram_ids ), '%s' ) );

    $query = $wpdb->prepare( 

        "SELECT media_id FROM {$wpdb->prefix}sbi_instagram_posts WHERE instagram_id IN ($placeholders)", 

        ...$instagram_ids 

    );

    $media_ids = $wpdb->get_results( $query );



    if ( empty( $media_ids ) ) {

        return new WP_Error( 'no_posts', 'No posts found for this feed', array( 'status' => 404 ) );

    }



    // Prepare response with custom image paths

    $upload_dir = wp_upload_dir(); // Get the WordPress uploads directory

    $posts_with_custom_paths = array();

    foreach ( $media_ids as $media ) {

        // $image_path = trailingslashit( $upload_dir['baseurl'] ) . 'sb-instagram-feed-images/' . $media->media_id . 'full.jpg'; // Adjust the path and filename as needed

        $image_path = trailingslashit( $upload_dir['baseurl'] ) . 'sb-instagram-feed-images/' . $media->media_id . 'full.webp'; // Adjust the path and filename as needed

       

        $posts_with_custom_paths[] = array(

            'media_id' => $media->media_id,

            'image_path' => $image_path,

        );

    }



    return rest_ensure_response( $posts_with_custom_paths );

}

